# importing Libraries
import os
import sys
import numpy as np
import pandas as pd
from sklearn.model_selection import train_test_split
from sklearn.linear_model import LinearRegression, Ridge, Lasso, ElasticNet
from sklearn.tree import DecisionTreeRegressor
from sklearn.ensemble import (
    RandomForestRegressor,
    GradientBoostingRegressor,
    AdaBoostRegressor,
)
from sklearn.svm import SVR
from sklearn.neighbors import KNeighborsRegressor
import xgboost as xgb
from sklearn.metrics import mean_absolute_error, mean_squared_error, r2_score
from sklearn.preprocessing import StandardScaler, RobustScaler, OneHotEncoder
from sklearn.compose import ColumnTransformer
import pickle
import warnings

warnings.filterwarnings("ignore")


def Regression(X, y, model="lr", scaler=None, cat=False):
    import warnings

    warnings.filterwarnings("ignore")

    # Train test split
    X_train, X_test, y_train, y_test = train_test_split(
        X, y, test_size=0.2, random_state=52
    )

    # One-hot encoding
    if cat:
        categorical_cols = X.select_dtypes(include=["O"]).columns.tolist()
        if categorical_cols:
            transformer = ColumnTransformer(
                transformers=[("ohe", OneHotEncoder(), categorical_cols)],
                remainder="passthrough",
            )
            X_train = transformer.fit_transform(X_train)
            X_test = transformer.transform(X_test)

    # Scaling the data
    if scaler == "standard":
        scaler = StandardScaler()
    elif scaler == "robust":
        scaler = RobustScaler()
    else:
        scaler = None

    if scaler is not None:
        X_train = scaler.fit_transform(X_train)
        X_test = scaler.transform(X_test)

    # Selecting the Model
    if model == "lr":
        reg = LinearRegression()
    elif model == "ridge":
        reg = Ridge()
    elif model == "lasso":
        reg = Lasso()
    elif model == "enet":
        reg = ElasticNet()
    elif model == "dt":
        reg = DecisionTreeRegressor()
    elif model == "rf":
        reg = RandomForestRegressor()
    elif model == "svm":
        reg = SVR()
    elif model == "knn":
        reg = KNeighborsRegressor()
    elif model == "gb":
        reg = GradientBoostingRegressor()
    elif model == "ada":
        reg = AdaBoostRegressor()
    elif model == "xbg":
        reg = xgb.XGBRegressor(n_estimators=100, learning_rate=0.05, random_state=42)
    else:
        return print("Invalid model name")

    # Fitting the Model
    reg.fit(X_train, y_train)
    y_pred = reg.predict(X_test)

    # Printing Metrics
    print(
        f" Mean Absolute Error :{mean_absolute_error(y_test, y_pred)},\
    \n \n Mean Squared Error :{mean_squared_error(y_test, y_pred)},\
    \n \n Root Mean Squared Error :{np.sqrt(mean_squared_error(y_test, y_pred))},\
    \n \n R2 Score :{r2_score(y_test, y_pred)}"
    )

    # Saving the Model
    model_dir = "model"
    if not os.path.exists(model_dir):
        os.mkdir(model_dir)

    filename = os.path.join(model_dir, f"{model}.pkl")
    with open(filename, "wb") as file:
        pickle.dump(reg, file)

    print(f" \n Model saved successfully in {filename}")



import nltk

nltk.download("stopwords")
from nltk.corpus import stopwords

stopwords.words("english")
nltk.download("punkt")
nltk.download("wordnet")
from nltk.tokenize import word_tokenize
from nltk.stem import PorterStemmer, WordNetLemmatizer
import re


def Text_preprocessing(
    text,
    stem=False,
    lemmatize=False,
    remove_html=True,
    remove_emoji=True,
    remove_special_chars=True,
    remove_extra_spaces=True,
    remove_url=True,
):
    # Removing HTML tags
    if remove_html:
        html_pattern = re.compile("<.*?>")
        text = html_pattern.sub("", text)

    # Removing URLs
    if remove_url:
        url_pattern = re.compile(r"https?://\S+|www\.\S+")
        text = url_pattern.sub("", text)

    # # Removing emojis
    # if remove_emoji:
    #     text = emoji.demojize(text)
    #     text = re.sub(r":[a-zA-Z_]+:", "", text)

    # Removing emojis
    if remove_emoji:
        emoji_pattern = re.compile(
            "["
            "\U0001F600-\U0001F64F"  # emoticons
            "\U0001F300-\U0001F5FF"  # symbols & pictographs
            "\U0001F680-\U0001F6FF"  # transport & map symbols
            "\U0001F1E0-\U0001F1FF"  # flags (iOS)
            "\U00002702-\U000027B0"  # other miscellaneous symbols
            "\U000024C2-\U0001F251"
            "]+",
            flags=re.UNICODE,
        )
    text = emoji_pattern.sub(r"", text)
    text = re.sub(r":[a-zA-Z_]+:", "", text)  # Remove emoji names within colons
    text = re.sub(r"\s+", " ", text)  # Remove extra spaces

    # Removing special characters
    if remove_special_chars:
        text = re.sub(r"[^a-zA-Z0-9\s]", "", text)

    # Removing extra spaces
    if remove_extra_spaces:
        text = re.sub(r"\s+", " ", text).strip()

    # Tokenization
    words = word_tokenize(text)

    # Stop word removal
    stop_words = set(stopwords.words("english"))
    filtered_words = [word for word in words if word.casefold() not in stop_words]

    # Stemming or Lemmatization
    if stem and not lemmatize:
        stemmer = PorterStemmer()
        stemmed_words = [stemmer.stem(word) for word in filtered_words]
        return " ".join(stemmed_words)

    elif not stem and lemmatize:
        lemmatizer = WordNetLemmatizer()
        lemmatized_words = [lemmatizer.lemmatize(word) for word in filtered_words]
        return " ".join(lemmatized_words)

    elif not stem and not lemmatize:
        return " ".join(filtered_words)

    else:
        raise ValueError("Only one of 'stem' or 'lemmatize' can be set to True.")



import setuptools

with open("README.md", "r", encoding="utf-8") as f:
    long_description = f.read()

__version__ = "0.0.13"

REPO_NAME = "mlfast"
AUTHOR_USER_NAME = "Abdul-Jaweed"
SRC_REPO = "mlfast"
AUTHOR_EMAIL = "jdgaming7320@gmail.com"

setuptools.setup(
    name=SRC_REPO,
    version=__version__,
    author=AUTHOR_USER_NAME,
    author_email=AUTHOR_EMAIL,
    description="its a python machine learning package",
    long_description=long_description,
    long_description_content="text/markdown",
    url=f"https://github.com/{AUTHOR_USER_NAME}/{REPO_NAME}",
    project_urls={
        "Bug Tracker": f"https://github.com/{AUTHOR_USER_NAME}/{REPO_NAME}/issues",
    },
    package_dir={"": "src"},
    packages=setuptools.find_packages(where="src"),
)



import openai
from gradio import gradio as gr


import warnings
warnings.filterwarnings("ignore") 


def Chatbot(api_key, role, deploy=False):
    # Set up the OpenAI API client
    openai.api_key = api_key

    # Initialize the conversation with the system message based on the role
    messages = [{"role": "system", "content": f"You are {role}. How can I assist you today?"}]

    def generate_response(input_text):
        # Get user input
        message = input_text

        # Add user message to the conversation
        messages.append({"role": "user", "content": message})

        # Check if user wants to quit
        if message.lower() == "bye":
            return "Goodbye!"

        # Generate a response from the chatbot
        response = openai.ChatCompletion.create(
            model="gpt-3.5-turbo",
            messages=messages
        )

        # Get the assistant's reply from the response
        reply = response.choices[0].message.content

        # Add assistant's reply to the conversation
        messages.append({"role": "assistant", "content": reply})

        return reply

    title = f"{role}"
    inputs = gr.inputs.Textbox(label="User Input")
    outputs = gr.outputs.Textbox(label="Assistant Reply")

    interface = gr.Interface(fn=generate_response, inputs=inputs, outputs=outputs, title=title)
    
    if deploy==True:
      interface.launch(share=True)
    else:
      interface.close()